package fr.natan.addressmicroservice.service.service;

import fr.natan.addressmicroservice.entity.Address;
import fr.natan.addressmicroservice.exceptions.AddressOneOrMoreFieldsEmptyException;
import fr.natan.addressmicroservice.exceptions.AddressnotFoundException;
import fr.natan.addressmicroservice.repository.AddressRepository;
import fr.natan.addressmicroservice.service.contract.AddressContract;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class AddressService  implements AddressContract {

    private final AddressRepository addressRepository;

    public AddressService(AddressRepository addressRepository) {
        this.addressRepository = addressRepository;
    }

    private Address addressNormalize(Address address){
        address.setNumHouse(address.getNumHouse().strip());
        address.setStreetName(address.getStreetName().strip());
        address.setPb(address.getPb().strip());
        address.setCity(address.getCity().strip());
        address.setCountry(address.getCountry().strip());

        return address;
    }

    private boolean isValidAddress(Address address){
        boolean isInvalid=false;
        if(address.getNumHouse().isEmpty()
                || address.getStreetName().isEmpty()
                || address.getPb().isEmpty()
                || address.getCity().isEmpty()
                || address.getCountry().isEmpty()
        ){
            isInvalid=true;
        }

        return isInvalid;

    }
    @Override
    public Address createAddress(Address address) throws AddressOneOrMoreFieldsEmptyException {
        address = addressNormalize(address);
       if(isValidAddress(address)){
           throw new AddressOneOrMoreFieldsEmptyException("one or more fields are empty");
       }
        return addressRepository.save(address);
    }

    @Override
    public List<Address> getAllAddresses() {
        return addressRepository.findAll();
    }

    @Override
    public Address getAddressByID(Long addressID) throws AddressnotFoundException {
        return addressRepository.findById(addressID).orElseThrow(
                ()-> new AddressnotFoundException("address not found")
        );
    }

    @Override
    public void deleteAddress(Long addressID) throws AddressnotFoundException {
        Address address = addressRepository.findById(addressID).orElseThrow(
                ()-> new AddressnotFoundException("address not found")
        );
        addressRepository.delete(address);
    }

    @Override
    public Address updateAddress(Long addressID, Address address) throws AddressnotFoundException {
        if(getAddressByID(addressID)==null){
            throw new AddressnotFoundException("address not found");
        }

        return addressRepository.save(address);
    }
}
